@@ -22,12 +22,7 @@
 #include "eztrace.h"
 
 #ifdef USE_MPI3
-static int MPI_Ibarrier_core (MPI_Comm c, MPI_Request *r)
-{
-  return libMPI_Ibarrier(c, r);
-}
-
-static void MPI_Ibarrier_epilog (MPI_Comm c, MPI_Fint *r)
+static void MPI_Ibarrier_prolog (MPI_Comm c, MPI_Fint *r)
 {
   int rank = -1;
   int size = -1;
@@ -36,9 +31,20 @@ static void MPI_Ibarrier_epilog (MPI_Comm c, MPI_Fint *r)
   EZTRACE_EVENT_PACKED_4 (EZTRACE_MPI_IBARRIER, (app_ptr)c, rank, size, (app_ptr)r);
 }
 
+static int MPI_Ibarrier_core (MPI_Comm c, MPI_Request *r)
+{
+  return libMPI_Ibarrier(c, r);
+}
+
+static void MPI_Ibarrier_epilog (MPI_Comm c, MPI_Fint *r)
+{
+  EZTRACE_EVENT_PACKED_1 (EZTRACE_MPI_STOP_IBARRIER, (app_ptr)r);
+}
+
 int MPI_Ibarrier (MPI_Comm c, MPI_Request *req)
 {
   FUNCTION_ENTRY;
+  MPI_Ibarrier_prolog(c, (MPI_Fint*)req);
   int ret = MPI_Ibarrier_core(c, req);
   MPI_Ibarrier_epilog(c, (MPI_Fint*)req);
 
@@ -51,8 +57,8 @@ void mpif_ibarrier_(MPI_Fint *c, MPI_Fint *r, int *error)
   MPI_Comm c_comm = MPI_Comm_f2c(*c);
   MPI_Request c_req = MPI_Request_f2c(*r);
 
+  MPI_Ibarrier_prolog(c_comm, r);
   *error = MPI_Ibarrier_core(c_comm, &c_req);
-
   *r= MPI_Request_c2f(c_req);
   MPI_Ibarrier_epilog(c_comm, r);
 }
