@@ -121,13 +121,13 @@ int trace_get_bits() {
  * A wrapper to test for errors
  * TODO: Correct error handling in that part of the code
  */
-inline long __ptrace(enum __ptrace_request request, pid_t pid, void *addr,
-                     void *data, const char *file, int line) {
+static inline long __ptrace(enum __ptrace_request request, pid_t pid, void *addr,
+		     void *data, const char *file, int line) {
   errno = 0;
   long result = ptrace(request, pid, addr, data);
   if (errno && result == -1) {
     pptrace_fubar("at %s:%d: ptrace(%lu, %d, %p, %p): %s", file, line,
-                  (long) request, pid, addr, data, strerror(errno));
+		  (long) request, pid, addr, data, strerror(errno));
   }
   return result;
 }
@@ -163,7 +163,7 @@ pid_t trace_run(char *path, char **argv, char **envp, int debug) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "execve(%s, ...)\n", path);
     execve(path, argv, envp);
     pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "Execution of %s failed\n",
-                  path);
+		  path);
     exit(-1);
   }
   pid_t tpid = debug ? child : getppid();
@@ -181,9 +181,9 @@ pid_t trace_run(char *path, char **argv, char **envp, int debug) {
   pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "execve(%s, ...) passed\n", path);
   if (WIFEXITED(status)) {
     pptrace_debug(
-        PPTRACE_DEBUG_LEVEL_VERBOSE,
-        "Process %d exited (probably due to execution of %s failed)\n", tpid,
-        path);
+	PPTRACE_DEBUG_LEVEL_VERBOSE,
+	"Process %d exited (probably due to execution of %s failed)\n", tpid,
+	path);
     return -1;
   }
   return tpid;
@@ -242,7 +242,7 @@ uint8_t trace_replace(pid_t child, word_uint addr, uint8_t newbyte) {
 
   data64 = (data64 & mask) | newvalue;
 pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "[" FORMAT_WORD_HEX "]\n", ARG_FORMAT_WORD(data64));
-	                          _ptrace(PTRACE_POKETEXT, child, (void* )addr64, (void* )data64);
+				  _ptrace(PTRACE_POKETEXT, child, (void* )addr64, (void* )data64);
 
   return (uint8_t) oldvalue;
 }
@@ -255,12 +255,12 @@ void trace_read(pid_t child, word_uint fromaddr, uint8_t* buffer, size_t length)
   for (curAddr = 0; curAddr < length; curAddr += word_size) {
     data = _ptrace(PTRACE_PEEKTEXT, child, (void*)(curAddr+fromaddr), NULL);
     memcpy(buffer + curAddr, (void*) (&data),
-           curAddr + word_size < length ? word_size : (length - curAddr));
+	   curAddr + word_size < length ? word_size : (length - curAddr));
   }
 }
 
 void trace_copy(pid_t child, word_uint fromaddr, word_uint toaddr,
-                size_t length) {
+		size_t length) {
   word_uint data;
   word_uint curAddr;
   int extraBytes = length % sizeof(word_uint);
@@ -273,7 +273,7 @@ void trace_copy(pid_t child, word_uint fromaddr, word_uint toaddr,
 pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Copying word [" FORMAT_WORD_HEX "] from address " FORMAT_WORD_POINTER " to address " FORMAT_WORD_POINTER "\n",
 		      ARG_FORMAT_WORD(data), curAddr+fromaddr, curAddr+toaddr);
 
-	            _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + toaddr), (void* )data);
+		    _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + toaddr), (void* )data);
   }
 
   if (extraBytes > 0) {
@@ -282,15 +282,15 @@ pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Copying word [" FORMAT_WORD_HEX "] from
     data = _ptrace(PTRACE_PEEKTEXT, child, (void*)(curAddr+fromaddr), NULL);
     data &= mask;
     pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Copying %d extra bytes (word [" FORMAT_WORD_HEX "] from address " FORMAT_WORD_POINTER " to address " FORMAT_WORD_POINTER ")\n",
-        extraBytes >> 3, ARG_FORMAT_WORD(data), curAddr+fromaddr, curAddr+toaddr);
+	extraBytes >> 3, ARG_FORMAT_WORD(data), curAddr+fromaddr, curAddr+toaddr);
     word_uint destData =
       _ptrace(PTRACE_PEEKTEXT, child, (void*)(curAddr+toaddr), NULL);
     pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Replacing [" FORMAT_WORD_HEX "] by ", ARG_FORMAT_WORD(destData));
     destData &= ~(mask);
     destData |= data;
 pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "[" FORMAT_WORD_HEX "]\n", ARG_FORMAT_WORD(destData));
-		                                                    _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + toaddr),
-            (void* )destData);
+								    _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + toaddr),
+	    (void* )destData);
   }
 }
 
@@ -327,7 +327,7 @@ void trace_write(pid_t child, word_uint addr, uint8_t* buffer, size_t length) {
     memcpy((void*) (&data), (void*) (buffer + curAddr), sizeof(word_uint));
 pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Inserting word [" FORMAT_WORD_HEX "] at address " FORMAT_WORD_POINTER "\n",
 		    ARG_FORMAT_WORD(data), curAddr+addr);
-                  _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + addr), (void* )data);
+		  _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + addr), (void* )data);
   }
   if (extraBytes > 0) {
     data = 0;
@@ -337,7 +337,7 @@ pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Inserting word [" FORMAT_WORD_HEX "] at
     // data >>= (64 - extraBytes);
     word_uint mask = ((1L << (extraBytes)) - 1);
     pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Inserting %d extra bytes (word [" FORMAT_WORD_HEX "] at address " FORMAT_WORD_POINTER ")\n", extraBytes >> 3,
-        ARG_FORMAT_WORD(data), curAddr+addr);
+	ARG_FORMAT_WORD(data), curAddr+addr);
     data &= mask;
     word_uint destData =
       _ptrace(PTRACE_PEEKTEXT, child, (void*)(curAddr+addr), NULL);
@@ -345,14 +345,15 @@ pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Inserting word [" FORMAT_WORD_HEX "] at
     destData &= ~(mask);
     destData |= data;
 pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "[" FORMAT_WORD_HEX "]\n", ARG_FORMAT_WORD(destData));
-		                                                    _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + addr), (void* )destData);
+								    _ptrace(PTRACE_POKETEXT, child, (void* )(curAddr + addr), (void* )destData);
   }
 }
 
 /* push a value on the stack.
  * return the value that is overwritten
  */
-word_uint do_push(pid_t child, word_uint *sp, word_uint value) {
+word_uint do_push(pid_t child __attribute__((unused)), word_uint *sp __attribute__((unused)),
+		  word_uint value __attribute__((unused))) {
 #if __arm__
 // ARMv7
   word_uint ret =-1;
@@ -372,7 +373,8 @@ word_uint do_push(pid_t child, word_uint *sp, word_uint value) {
  * has been overwritten by do_push
  * returns the popped value
  */
-word_uint do_pop(pid_t child, word_uint *sp, word_uint prev_value) {
+word_uint do_pop(pid_t child __attribute__((unused)), word_uint *sp __attribute__((unused)),
+		 word_uint prev_value __attribute__((unused))) {
 #if __arm__
   // ARMv7
   word_uint ret;
@@ -415,9 +417,6 @@ _ptrace(PTRACE_GETREGS, child, NULL, &regs);
 REG_STRUCT saved_regs;
 _ptrace(PTRACE_GETREGS, child, NULL, &saved_regs);
 
-word_uint prev_val1 = -1;
-word_uint prev_val2 = -1;
-
 #if __WORDSIZE == 64
 if(ptrace_bits == 32) {
 #endif
@@ -435,6 +434,10 @@ if(ptrace_bits == 32) {
  sp+16 = arg6 => offset = 0
  return r0 = return value = addr
  */
+
+word_uint prev_val1 = -1;
+word_uint prev_val2 = -1;
+
 regs.REG_R7 = 192; // __NR_mmap2 32-bits
 regs.REG_R0 = addr;
 regs.REG_R1 = length;
@@ -506,7 +509,7 @@ int size = 2;
 #endif
 for (i = 0; i < size; i++)
 #if __arm__
-// ARMv7 
+// ARMv7
   opcode[i] = trace_replace(child, saved_regs.REG_PC + i, opcode[i]);
 #elif __x86_64__
 // x86_64
@@ -546,7 +549,7 @@ return get_retval(regs);
 }
 
 char *trace_get_zstring(pid_t child, word_uint addr) {
-int size = 1024;
+unsigned int size = 1024;
 unsigned int index = 0;
 if (addr == 0)
   return NULL;
@@ -584,7 +587,7 @@ case SYSCALL_ARGTYPE_ZSTRING:
   }
   buf = trace_get_zstring(child, reg);
   pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, " \"%s\" (%p) == \"%s\"?", buf,
-                (void*) reg, ptr);
+		(void*) reg, ptr);
   r = strcmp(ptr, buf);
 
   free(buf);
@@ -639,7 +642,7 @@ if (ptrace_bits == 32) {
   _sma_TEST_ARG(child, type, args, buf[0]);
   _sma_TEST_ARG(child, type, args, buf[1]);
 #elif __x86_64__
-// x86_64        
+// x86_64
   if (syscall == TRACE_SYSCALL(mmap)) {
     uint32_t buf[6];
     trace_read(child, regs.REG_BX, (uint8_t*) buf, 24);
@@ -682,17 +685,17 @@ return 1;
 #define TEST_SYSCALL(syscalls, syscall) (((syscall != 0) || (syscall == syscalls)) && (((syscalls & 0xffff) == syscall) || (((syscalls >> 16) & 0xffff) == syscall) || (((syscalls >> 32) & 0xffff) == syscall) || (((syscalls >> 48) & 0xffff) == syscall)))
 #endif
 int trace_wait_syscall(pid_t child, word_uint* retval, unsigned long syscall,
-                       ...) {
+		       ...) {
 // Wait for syscall and return the return value of the first encountered
 int status;
 REG_STRUCT regs;
-int in_syscall = 0;
+unsigned int in_syscall = 0;
 int in_execve = 0;
 va_list args;
 
 pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG,
-              "Looking for syscalls "SYSCALLS_FORMAT"\n",
-              SYSCALLS_PRINT_ARGS(syscall));
+	      "Looking for syscalls "SYSCALLS_FORMAT"\n",
+	      SYSCALLS_PRINT_ARGS(syscall));
 
 for (status = trace_syscall(child); !WIFEXITED(status) && !WIFSIGNALED(status);
     status = trace_syscall(child)) {
@@ -704,63 +707,63 @@ for (status = trace_syscall(child); !WIFEXITED(status) && !WIFSIGNALED(status);
       if (!in_syscall) {
 #if __arm__
 // ARMv7
-        in_syscall = regs.REG_R7;
+	in_syscall = regs.REG_R7;
 #elif __x86_64__
 // x86_64
-        in_syscall = regs.REG_ORIG_AX;
+	in_syscall = regs.REG_ORIG_AX;
 #endif
-        pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Entering syscall %d\n",
-                      in_syscall);
+	pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Entering syscall %d\n",
+		      in_syscall);
 
-        if (in_syscall == TRACE_SYSCALL(execve))
-          in_execve = 1; // execve generate 3 sigtraps: one at call, one at program loading and one at syscall return
-        if (TEST_SYSCALL(syscall, in_syscall)) {
-          pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG, "Found syscall entry %d with registers "REG_FORMAT"\n", in_syscall, REG_FORMAT_VALUES(regs));
-          // We are in the good syscall but have we the good arguments?
-          va_start(args, syscall);
-          pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Checking arguments: ");
-          if (!syscall_match_args(child, in_syscall, &args, regs)) {
-            in_syscall = -in_syscall; // Arguments doesn't match
-            pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, " failed!\n");
-          } else {
-            pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, " ok\n");
-          }
-          va_end(args);
-        }
+	if (in_syscall == TRACE_SYSCALL(execve))
+	  in_execve = 1; // execve generate 3 sigtraps: one at call, one at program loading and one at syscall return
+	if (TEST_SYSCALL(syscall, in_syscall)) {
+	  pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG, "Found syscall entry %d with registers "REG_FORMAT"\n", in_syscall, REG_FORMAT_VALUES(regs));
+	  // We are in the good syscall but have we the good arguments?
+	  va_start(args, syscall);
+	  pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Checking arguments: ");
+	  if (!syscall_match_args(child, in_syscall, &args, regs)) {
+	    in_syscall = -in_syscall; // Arguments doesn't match
+	    pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, " failed!\n");
+	  } else {
+	    pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, " ok\n");
+	  }
+	  va_end(args);
+	}
       } else if (in_execve) {
-        pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL,
-                      "execve has stopped after loading the new program\n");
-        in_execve = 0;
+	pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL,
+		      "execve has stopped after loading the new program\n");
+	in_execve = 0;
       } else {
-        // Exiting a syscall
-        pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Exiting syscall %d\n",
-                      in_syscall);
-        if (TEST_SYSCALL(syscall, in_syscall)) {
-          pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG, "Found syscall %d with registers "REG_FORMAT"\n", in_syscall, REG_FORMAT_VALUES(regs));
+	// Exiting a syscall
+	pptrace_debug(PPTRACE_DEBUG_LEVEL_ALL, "Exiting syscall %d\n",
+		      in_syscall);
+	if (TEST_SYSCALL(syscall, in_syscall)) {
+	  pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG, "Found syscall %d with registers "REG_FORMAT"\n", in_syscall, REG_FORMAT_VALUES(regs));
 
-          if (retval != NULL)
-            *retval = get_retval(regs);
+	  if (retval != NULL)
+	    *retval = get_retval(regs);
 
-          return 0;
-        }
-        in_syscall = 0;
+	  return 0;
+	}
+	in_syscall = 0;
       }
     } else if (signal == SIGSEGV || signal == SIGKILL || signal == SIGILL) {
       // Signal that terminates the process
       pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG,
-                    "Syscalls "SYSCALLS_FORMAT" not found\n",
-                    SYSCALLS_PRINT_ARGS(syscall));
+		    "Syscalls "SYSCALLS_FORMAT" not found\n",
+		    SYSCALLS_PRINT_ARGS(syscall));
       return -1;
     } else
       pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG,
-                    "Stopped with unknown signal %d\n", signal);
+		    "Stopped with unknown signal %d\n", signal);
   } else
     pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG,
-                  "Waitpid returned with the process unstopped\n");
+		  "Waitpid returned with the process unstopped\n");
 }
 pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG,
-              "Syscalls "SYSCALLS_FORMAT" not found\n",
-              SYSCALLS_PRINT_ARGS(syscall));
+	      "Syscalls "SYSCALLS_FORMAT" not found\n",
+	      SYSCALLS_PRINT_ARGS(syscall));
 return -1;
 }
 
