@@ -20,6 +20,7 @@
 #include <errors.h>
 #include <hijack.h>
 #include <pptrace.h>
+#include <assert.h>
 
 #include <string.h>
 #include <stdlib.h>
@@ -27,6 +28,7 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <fcntl.h>
+#include "eztrace_config.h"
 
 // To store informations about hijack to install
 typedef struct __pptrace_internal_hijack {
@@ -165,8 +167,8 @@ void pptrace_free_hijack(pptrace_internal_hijack* hijack) {
 }
 
 pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
-                                            char *library, void *lib,
-                                            char *params) {
+					    char *library, void *lib,
+					    char *params) {
   char *params_copy;
   char *function;
   char *interceptee;
@@ -187,8 +189,8 @@ pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
   ;
 
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE,
-                "Got order to intercept %s with %s using %s\n", function,
-                interceptor, interceptee);
+		"Got order to intercept %s with %s using %s\n", function,
+		interceptor, interceptee);
   if (!hijack) {
     pptrace_error("Allocation failed");
     free(params_copy);
@@ -224,8 +226,8 @@ pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
   hijack->funSym = get_symbol(bin->binary, hijack->function);
   if (!hijack->funSym) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO,
-                  "Failed to find symbol %s in binary %s, ignoring!\n",
-                  hijack->function, library);
+		  "Failed to find symbol %s in binary %s, ignoring!\n",
+		  hijack->function, library);
     pptrace_free_hijack(hijack);
     return NULL;
   }
@@ -233,7 +235,7 @@ pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
   hijack->origSym = get_symbol(lib, hijack->orig_symbol);
   if (!hijack->origSym) {
     pptrace_error("Failed to find symbol %s in binary %s", hijack->orig_symbol,
-                  library);
+		  library);
     pptrace_free_hijack(hijack);
     return NULL;
   }
@@ -241,7 +243,7 @@ pptrace_internal_hijack* pptrace_get_hijack(pptrace_internal_binary *bin,
   hijack->replSym = get_symbol(lib, hijack->replace_symbol);
   if (!hijack->replSym) {
     pptrace_error("Failed to find symbol %s in binary %s",
-                  hijack->replace_symbol, library);
+		  hijack->replace_symbol, library);
     pptrace_free_hijack(hijack);
     return NULL;
   }
@@ -259,8 +261,8 @@ void pptrace_free_hijacks(pptrace_internal_hijack** hijacks) {
 }
 
 pptrace_internal_hijack** pptrace_get_hijacks(pptrace_internal_binary *bin,
-                                              char *library, void *lib,
-                                              char **params) {
+					      char *library, void *lib,
+					      char **params) {
   int s, i, j;
   for (s = 0; params[s] != NULL; s++)
     ;
@@ -284,21 +286,53 @@ pptrace_internal_hijack** pptrace_get_hijacks(pptrace_internal_binary *bin,
   return hijacks;
 }
 
-pptrace_internal_hijack** pptrace_load_hijacks(pptrace_internal_binary *bin,
-                                               char *library, void *lib) {
-  void *bin_bin = open_binary(library);
-  char** params = read_zstring_array(bin_bin, PPTRACE_HIJACK_FUNCTION_NAME);
-  close_binary(bin_bin);
+void __pptrace_get_module_name(const char* libname,
+			       char *module_name,
+			       size_t buffer_size) {
+  #define LIBEZTRACE_PREFIX "libeztrace-autostart-"
+  memset(module_name, '\0', buffer_size);
+  /* search for "libeztrace-autostart-" */
+  char* res = strstr(libname, LIBEZTRACE_PREFIX);
+  if(res) {
+    /* found it */
+    res = &res[strlen(LIBEZTRACE_PREFIX)];
+    /* now search for the suffix (".so" / ".dynlib") */
+    char* suffix = strstr(res, DYNLIB_SUFFIX);
+    if(suffix) {
+      /* found it */
+      assert(suffix-res < buffer_size);
+      strncpy(module_name, res, suffix-res);
+    }
+  }
+}
 
+pptrace_internal_hijack** pptrace_load_hijacks(pptrace_internal_binary *bin,
+					       char *library, void *lib) {
+  void *bin_bin = open_binary(library);
+  char module_name[1024];
+  __pptrace_get_module_name(library, module_name, 1024);
+
+  char hijack_symbol[1024];
+  __get_pptrace_symbol_external(hijack_symbol, 1024, module_name);
+
+  char** params = read_zstring_array(bin_bin, hijack_symbol);
   if (!params) {
-    pptrace_error(
-        "%s library does not contain symbol %s, probably wrong library format",
-        library, PPTRACE_HIJACK_FUNCTION_NAME);
-    return NULL;
+    __get_pptrace_symbol_external(hijack_symbol, 1024, "");
+    params = read_zstring_array(bin_bin, hijack_symbol);
+    if(!params) {
+      pptrace_error("%s library does not contain symbol %s, probably wrong library format",
+		    library, hijack_symbol);
+      close_binary(bin_bin);
+
+      return NULL;
+    }
   }
 
+  close_binary(bin_bin);
+
+
   pptrace_internal_hijack **result = pptrace_get_hijacks(bin, library, lib,
-                                                         params);
+							 params);
   free_zstring_array(params);
   return result;
 }
@@ -317,7 +351,7 @@ void pptrace_free_library(pptrace_internal_library *lib) {
 }
 
 void pptrace_insert_library(pptrace_internal_binary* binary,
-                            pptrace_internal_library* lib) {
+			    pptrace_internal_library* lib) {
   lib->next = NULL;
   lib->prev = binary->last;
   if (binary->last) {
@@ -346,7 +380,7 @@ int pptrace_load_module(void *bin, char *library) {
     return r;
 
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Fetching hijacks of library %s\n",
-                library);
+		library);
   pptrace_internal_binary* binary = (pptrace_internal_binary*) bin;
   pptrace_internal_library* lib = binary->last;
   void *libr = open_binary(library);
@@ -366,7 +400,7 @@ int pptrace_load_module(void *bin, char *library) {
   lib->baseaddr = 0;
 
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Hijacks of library %s fetched\n",
-                library);
+		library);
   return 0;
 }
 
@@ -404,10 +438,10 @@ int pptrace_add_preload(void *bin, char *library) {
 }
 
 char* pptrace_create_preload(pptrace_internal_library *first) {
-  const static char* ldpreload = "LD_PRELOAD=";
+  const char* ldpreload = "LD_PRELOAD=";
   char *result = (char*) malloc(1024);
-  int size = 1024;
-  int nsize = strlen(ldpreload);
+  unsigned int size = 1024;
+  unsigned int nsize = strlen(ldpreload);
   strcpy(result, ldpreload);
   while (first != NULL) {
     while (size - nsize < strlen(first->library) + 2) {
@@ -436,7 +470,7 @@ char* pptrace_create_preload(pptrace_internal_library *first) {
 }
 
 pid_t pptrace_launch_preload(char *path, char **argv, char **envp,
-                             pptrace_internal_library *first, int debug) {
+			     pptrace_internal_library *first, int debug) {
   // Adding LD_PRELOAD=libraries in envp
   int i, size = 0;
   while (envp[size] != NULL)
@@ -494,7 +528,7 @@ pptrace_internal_library* pptrace_search_library(
   while (result != NULL) {
     int len = libnamelen - strlen(result->library);
     pptrace_debug(PPTRACE_DEBUG_LEVEL_DEBUG, "Comparing %s and %s\n",
-                  result->library, libname + len);
+		  result->library, libname + len);
     if (len >= 0 && strcmp(result->library, libname + len) == 0) // the searched library ends with the short name of the lib
       return result;
     result = result->next;
@@ -503,7 +537,7 @@ pptrace_internal_library* pptrace_search_library(
 }
 
 void pptrace_apply_library_baseaddr(pptrace_internal_library *lib,
-                                    zzt_word baseaddr) {
+				    zzt_word baseaddr) {
   int i;
   lib->baseaddr = baseaddr;
   if (lib->hijacks) {
@@ -518,18 +552,18 @@ int pptrace_wait_library(pid_t child, pptrace_internal_binary* binary) {
   char *library;
   // Wait for dlopen(argv[1])
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE,
-                "Waiting for open(*, O_RDONLY) (syscall %d)... ",
-                TRACE_SYSCALL(open));
+		"Waiting for open(*, O_RDONLY) (syscall %d)... ",
+		TRACE_SYSCALL(open));
   word_uint fd;
   int r = trace_wait_syscall(child, &fd, TRACE_SYSCALL(open),
-                             SYSCALL_ARGTYPE_ZSTRING_RETURN, &library,
-                             SYSCALL_ARGTYPE_IGNORE, SYSCALL_ARGTYPE_END);
+			     SYSCALL_ARGTYPE_ZSTRING_RETURN, &library,
+			     SYSCALL_ARGTYPE_IGNORE, SYSCALL_ARGTYPE_END);
   if (r < 0 || library == NULL) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "failed!\n");
     return -1;
   }
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "ok (fd = %d, library = %s)\n", fd,
-                library);
+		library);
 
   if ((int) fd < 0) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "open() failed ==> ignoring\n");
@@ -540,7 +574,7 @@ int pptrace_wait_library(pid_t child, pptrace_internal_binary* binary) {
   free(library);
   if (!lib || lib->baseaddr) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE,
-                  "Not an awaited library, ignoring\n");
+		  "Not an awaited library, ignoring\n");
     return 0;
   }
 
@@ -550,22 +584,22 @@ int pptrace_wait_library(pid_t child, pptrace_internal_binary* binary) {
       fd, MMAP_SYSCALLS);
   word_uint baseaddr = 0;
   r = trace_wait_syscall(child, &baseaddr, MMAP_SYSCALLS,
-                         SYSCALL_ARGTYPE_IGNORE, SYSCALL_ARGTYPE_IGNORE,
-                         SYSCALL_ARGTYPE_INT,
-                         (word_uint) (PROT_READ | PROT_EXEC),
-                         SYSCALL_ARGTYPE_IGNORE, SYSCALL_ARGTYPE_INT,
-                         (word_uint) fd, SYSCALL_ARGTYPE_IGNORE,
-                         SYSCALL_ARGTYPE_END);
+			 SYSCALL_ARGTYPE_IGNORE, SYSCALL_ARGTYPE_IGNORE,
+			 SYSCALL_ARGTYPE_INT,
+			 (word_uint) (PROT_READ | PROT_EXEC),
+			 SYSCALL_ARGTYPE_IGNORE, SYSCALL_ARGTYPE_INT,
+			 (word_uint) fd, SYSCALL_ARGTYPE_IGNORE,
+			 SYSCALL_ARGTYPE_END);
   if (baseaddr == 0 || r < 0) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "failed!\n");
     return -1;
   }
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "ok (addr = %016lx)\n", baseaddr);
   pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE,
-                "Waiting for close(%d) (syscall %d)... ", fd,
-                TRACE_SYSCALL(close));
+		"Waiting for close(%d) (syscall %d)... ", fd,
+		TRACE_SYSCALL(close));
   if (trace_wait_syscall(child, NULL, TRACE_SYSCALL(close), SYSCALL_ARGTYPE_INT,
-                         fd, SYSCALL_ARGTYPE_END) < 0) {
+			 fd, SYSCALL_ARGTYPE_END) < 0) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_VERBOSE, "failed!\n");
     return -1;
   }
@@ -593,20 +627,20 @@ int pptrace_install_hijacks(pid_t child, pptrace_internal_binary* binary) {
   for (lib = binary->first; lib != NULL; lib = lib->next) {
     if (lib->hijacks) {
       for (i = 0; lib->hijacks[i] != NULL; i++) {
-        if (lib->hijacks[i]->funSym != NULL
-          && lib->hijacks[i]->funSym->symbol_offset) {
-          // Replacing symbol only when the symbol is not in a dynamic library
-          if (hijack(binary->binary, child, lib->hijacks[i]->funSym,
-                     lib->hijacks[i]->origSym, lib->hijacks[i]->replSym) < 0) {
-            fprintf(stderr, "Failed to install hijack of symbol %s\n",
+	if (lib->hijacks[i]->funSym != NULL
+	  && lib->hijacks[i]->funSym->symbol_offset) {
+	  // Replacing symbol only when the symbol is not in a dynamic library
+	  if (hijack(binary->binary, child, lib->hijacks[i]->funSym,
+		     lib->hijacks[i]->origSym, lib->hijacks[i]->replSym) < 0) {
+	    fprintf(stderr, "Failed to install hijack of symbol %s\n",
 		    lib->hijacks[i]->function);
-          } else {
+	  } else {
 	    nb_installed ++;
-            pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO,
-                          "Function %s instrumented.\n",
-                          lib->hijacks[i]->function);
-          }
-        }
+	    pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO,
+			  "Function %s instrumented.\n",
+			  lib->hijacks[i]->function);
+	  }
+	}
       }
     }
   }
@@ -623,9 +657,9 @@ int pptrace_run(void *bin, char **argv, char **envp) {
   pptrace_internal_binary* binary = (pptrace_internal_binary*) bin;
   trace_set_bits(get_binary_bits(binary->binary));
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Running binary %s... ",
-                binary->name);
+		binary->name);
   pid_t child = pptrace_launch_preload(binary->name, argv, envp, binary->first,
-                                       (binary->debugger != NULL) ? 1 : 0);
+				       (binary->debugger != NULL) ? 1 : 0);
   if (child <= 0) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "failed!\n");
     pptrace_error("Failed to run binary %s", binary->name);
@@ -637,7 +671,7 @@ int pptrace_run(void *bin, char **argv, char **envp) {
   pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO, "Getting libraries' addresses\n");
   if (pptrace_wait_libraries(child, binary)) {
     pptrace_error(
-        "Failed to wait for open of the library, exiting after detaching child process...");
+	"Failed to wait for open of the library, exiting after detaching child process...");
     pptrace_free_binary(binary);
     return -1;
   }
@@ -646,24 +680,24 @@ int pptrace_run(void *bin, char **argv, char **envp) {
   if (pptrace_install_hijacks(child, binary) < 0) {
     trace_detach(child);
     pptrace_error(
-        "Failed to install hijacks, exiting after detaching child process...");
+	"Failed to install hijacks, exiting after detaching child process...");
     pptrace_free_binary(binary);
     return -1;
   }
 
   if (binary->debugger != NULL) {
     pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO,
-                  "Giving the control to the debugger %s\n",
-                  binary->debugger[0]);
+		  "Giving the control to the debugger %s\n",
+		  binary->debugger[0]);
     char pid[10];
     int i;
     snprintf(pid, 10, "%d", child);
     pid[9] = 0;
     for (i = 0; binary->debugger[i] != NULL; i++) {
       if (strcmp(binary->debugger[i], "{pid}") == 0) {
-        binary->debugger[i] = pid;
+	binary->debugger[i] = pid;
       } else if (strcmp(binary->debugger[i], "{name}") == 0) {
-        binary->debugger[i] = binary->name;
+	binary->debugger[i] = binary->name;
       }
     }
     char *debugger = get_program_path(binary->debugger[0]);
@@ -676,7 +710,7 @@ int pptrace_run(void *bin, char **argv, char **envp) {
   } else {
     pptrace_free_binary(binary);
     pptrace_debug(PPTRACE_DEBUG_LEVEL_INFO,
-                  "Detaching and waiting the end of the process\n");
+		  "Detaching and waiting the end of the process\n");
     trace_detach(child);
     trace_wait(child);
   }
